#!/bin/sh

blink1="/usr/local/bin/blink1-tool"

SPACE="#000000,0.25,0,"
RED="#ff0000,0.25,0,${SPACE}"
GREEN="#00ff00,0.25,0,${SPACE}"
BLUE="#0000ff,0.25,0,${SPACE}"
MAGENTA="#ff00ff,0.25,0,${SPACE}"
WHITE="#ffffff,0.25,0,${SPACE}"
OFF="#000000,0.25,0,${SPACE}"

blink_clear_pattern() {
  pkill blink1-tool; $blink1 --off >/dev/null 2>&1
}

blink_play_pattern() {
  # usage: blink_play_pattern "${RED}${GREEN}${BLUE}"
  blink_clear_pattern
  BEGIN="'0,"; END="#000000,1.5,0'"
  p=${BEGIN}${1}${END}
  $blink1 --playpattern "$p" -b 64 -m 0 -q >/dev/null 2>&1 &
}

# blink(1) status results for each step
res_initialize="${RED}"
res_backup="${RED}"
res_scrub="${OFF}"
res_terminate="${RED}"
res_status="\${res_initialize}\${res_backup}\${res_scrub}\${res_terminate}"

blink_play_status() {
  blink_play_pattern "$(eval echo "${res_status}")"
}

log() {
  logger "$(basename "$0"): ${1}" >/dev/null 2>&1
}

run() {
  # redirect stdout and stderr from command to log and return $?
  output=$("$@" 2>&1); error=$?; log "$output"; return $error
}

initialize() {
  # abort if the backup drive is already mounted
  if run geli status gpt/backup.eli || \
     run zpool list backup
  then
    log "the backup drive is already mounted"; return 2
  fi

  # mount the backup drive
  if run geli attach -j /usr/local/etc/backup.passwd /dev/gpt/backup && \
     run zpool import backup
  then
    res_initialize="${GREEN}"; return 0
  fi

  return 1
}

scrub_start() {
  # run zpool scrub on Saturdays
  if [ "$(date +"%u")" -eq 6 ]; then
    log "starting zpool scrub backup"
	run zpool scrub backup
    return 0
  fi
  return 1
}

scrub_wait() {
  blink_play_pattern "${WHITE}"
  while zpool status backup | \
      grep -E " scan: +scrub in progress since " >/dev/null 2>&1
  do
    log "waiting for zpool scrub backup"
    sleep 60
  done
  log "finished zpool scrub backup"
  blink_clear_pattern
  results=$(zpool status backup | \
            sed -n 's/^.*scan: scrub repaired *//p' | \
            sed -n 's/ *in .*with */;/p' | \
            sed -n 's/ *errors on .*$//p')
  repaired=$(echo "$results" | cut -d ';' -f1)
  errors=$(echo "$results" | cut -d ';' -f2)
  if [ "$repaired" -eq 0 ] && [ "$errors" -eq 0 ]; then
    res_scrub="${GREEN}"
	return 0
  fi

  log "zfs scrub repaired: ${repaired} with errors: ${errors}"
  res_scrub="${MAGENTA}"
  return 1
}

backup() {
  # abort if the backup drive is over 80% full
  if [ "$(zpool list -H -o capacity backup | sed -r 's/%//g')" -gt 80 ]; then
    log "backup disk is low on space"
    res_backup="${MAGENTA}"
	return 1
  fi

  # start zpool scrub
  scrub_start; scrub=$?

  # track backup attempts and successes
  self_tries=0; self_successes=0
  bsdclient_tries=0; bsdclient_successes=0

  # repeat until 3AM tomorrow
  end=$(date -v+1d +"%Y%m%d030000")
  until [ "$(date +"%Y%m%d%H%M%S")" -gt "$end" ]
  do
    # perform backup commands
    blink_play_pattern "${BLUE}"
	
	# back up self
    self_tries=$((self_tries+1))
    if run /usr/local/sbin/zxfer -dFkPv -g 376 \
      -I com.sun:auto-snapshot \
	  -R zroot backup/"$(hostname -s)"
	then
      self_successes=$((self_successes+1))
	fi

	# back up bsdclient
    bsdclient_tries=$((bsdclient_tries+1))
    if run /usr/local/sbin/zxfer -dFkPv -g 376 \
      -I com.sun:auto-snapshot \
	  -O 'backup@192.168.1.108 -i /usr/home/backup/.ssh/id_rsa' \
	  -R zroot backup/bsdclient
	then
      bsdclient_successes=$((bsdclient_successes+1))
	fi

    # sleep 15 minutes between backup cycles
    sleep 900
  done

  # report backup results
  log "$(hostname -s) backed up ${self_successes}/${self_tries} attempts"
  log "bsdclient backed up ${bsdclient_successes}/${bsdclient_tries} attempts"
  if [ $self_successes -gt 0 ] && [ $bsdclient_successes -gt 0 ]; then
    res_backup="${GREEN}"
  fi

  # wait for zfs scrub to finish
  [ "$scrub" -eq 0 ] && scrub_wait

  return 0
}

terminate() {
  log "dismounting backup drive"
  while zpool list backup; do
    zpool export backup && break
    sleep 1
  done
  while geli status gpt/backup.eli; do
    geli detach gpt/backup.eli && break
    sleep 1
  done

  res_terminate="${GREEN}"
}

if [ "$(pgrep -fl "$(basename "$0")" | wc -l)" -gt 0 ]; then
  # abort if the script is already running
  instances="$(pgrep -fl "$(basename "$0")")"
  log "already running ${instances}"
  echo "$(basename "$0"): already running ${instances}"
  exit 1
elif [ "$1" = "--on-insert" ]; then
  # process on-insert
  initialize; error=$?
  [ $error -eq 2 ] && { blink_play_status; exit; }
  [ $error -eq 0 ] && backup
  blink_play_status; terminate; blink_play_status
elif [ "$1" = "--on-eject" ]; then
  # process on-eject
  blink_clear_pattern
fi
